<?php
/* 
Description		:: this class handles all DB related operation functions.  */
class db 
{
	private $host;
	private $user;
	private $pass;
	private $dbname;
	private $dbh;
	private $stmt;
	
	public $dbConnected = false;
	
	//initialize PDO connection
	public function __construct()
	{
		// Set DSN
		global $dbDetails;
		$this->host 	= $dbDetails['hostName'];  //  host name from  config file(config.php)
		$this->user 	= $dbDetails['userName']; //  database user name from config file(config.php)
		$this->pass 	= $dbDetails['userPassword']; // database password from config file(config.php)
		$this->dbname 	= $dbDetails['dbName'];   //  database name from config file(config.php)
		$this->Connect();
	}
	
	//connect to PDO
	public function Connect()
	{
		if(!$this->dbConnected)
		{
			$dsn = 'mysql:host='.$this->host.';dbname='.$this->dbname;
			// Set options
			$options = array(
				PDO::ATTR_PERSISTENT    => true,
				PDO::ATTR_ERRMODE       => PDO::ERRMODE_EXCEPTION,
				PDO::MYSQL_ATTR_INIT_COMMAND => "SET NAMES utf8"
			);
			try
			{
				$this->dbh = new PDO($dsn, $this->user, $this->pass, $options);
				# Connection succeeded, set the boolean to true.
				if($this->dbh)
				$this->dbConnected = true;
			}
			catch (PDOException $e)
			{
				# Write into log
				echo $this->ExceptionLog($e->getMessage());
				die();
			}
		}
	}
	
	// prepare PDO statement
	public function prepareStatement($query)
	{
		try
		{
			$this->stmt = $this->dbh->prepare($query); 
		} 
		catch(PDOException $e)
		{
			$this->error = $e->getMessage();
		}
		return $this->stmt;
	}
	
	// prepare and execute PDO statement
	public function prepareConditionStatement($query,$condnValues=array())
	{
		//echo $query;
		//print_r($condnValues);
		try
		{ 
			$this->stmt = $this->dbh->prepare($query); 
			if(count($condnValues)>0)
			{
				$this->stmt->execute($condnValues); 
			}
			else
			{
				$this->stmt->execute();
			}
		} 
		catch(PDOException $e)
		{
			echo $e->getMessage();
		}
	}
	
	// prepare and execute PDO statement  and returns whole result set
	public function resultSetAll($query,$condnValues=array())
	{  
		try
		{  
			$this->stmt = $this->dbh->prepare($query); 
			if(count($condnValues)>0)
			{
				$this->stmt->execute($condnValues); 
			}
			else
			{
				$this->stmt->execute(); 
			}

			return $this->stmt->fetchAll(PDO::FETCH_ASSOC);
		} 
		catch(PDOException $e)
		{
			$this->error = $e->getMessage();
			echo $this->error ;
		}
	}
	
	// prepare and execute PDO statement  for fetching a single record
	public function resultSetSingle($query,$condnValues=array())
	{ 
		try
		{
			$this->stmt = $this->dbh->prepare($query); 
			if(count($condnValues)>0)
			{
				$this->stmt->execute($condnValues);
			}
			else
			{
				$this->stmt->execute();
			}
			return $this->stmt->fetch(PDO::FETCH_ASSOC);
		} 
		catch(PDOException $e)
		{
			$this->error = $e->getMessage();
		}
	}
	
	// bind values
	public function bind($param, $value, $type = null)
	{ 
		if (is_null($type))
		{
			switch (true) 
			{
				case is_int($value):
					$type = PDO::PARAM_INT;
					break;
				case is_bool($value):
					$type = PDO::PARAM_BOOL;
					break;
				case is_null($value):
					$type = PDO::PARAM_NULL;
					break;
				default:
					$type = PDO::PARAM_STR;
			}
		}
		$this->stmt->bindValue($param, $value, $type);
	}
	
	
	// prepare statement for insert. parameters passed are : table name, assciative array containing fields and values to be inserted
	
	public function prepareInsertStatement($table, $arFieldsValues)
	{
		$fields	=	array_keys($arFieldsValues);
		$values	=	array_values($arFieldsValues);
				
		$formatedValues	=	array();
		$formatedValues1	=	array();
		foreach($values as $key=>$val)
		{
			if(!is_numeric($val))
			{
				if(strcmp($val,"NOW()") == 0)
					{$vala	=	$val;$valu	= $val; $formatedValues[]	=	$vala; }
				else
					{ $vala	=	"?"; 
					if($fields[$key]=='map_html')
					{
					 $valu	=	trim($val);
					}
					else
					{
					 $valu	=	addslashes(trim($val));
					}
					 $formatedValues[]	=	$vala; $formatedValues1[] = $valu; }
			}
			else
			{
			  $vala	=	"?"; $valu = $val;$formatedValues[]	=	$vala; $formatedValues1[] = $valu;
			}
			
		}
		
		$sql	=	"INSERT INTO ".$table." (";
		$sql	.=	implode(", ",$fields).") ";
		$sql	.=	"VALUES( ";
		$sql	.=	implode(", ",$formatedValues);
		$sql	.=	")";
		$this->prepareConditionStatement($sql,$formatedValues1);
		return $this->dbh->lastInsertId();
	}
	
	/* prepare statement for update. parameters passed are : table name, assciative array containing fields and values to be updated,condition for update, array containing values on conditions */
	function prepareUpdateStatement($table, $arFieldsValues, $strCondition='',$conditionValues)
	{
		$formatedValues	=	array();
		$formatedValues1	=	array();
		foreach($arFieldsValues as $key => $val)
		{
			if(!is_numeric($val))
			{
				if(strcmp($val,"NOW()") == 0)
					{$vala	=	$val;$valu	= $val; $formatedValues[]	=	"$key = $vala"; }
				else
					{ $vala	=	"?"; 
					
					if($key=='map_html')
					{
					 $valu	=	trim($val);
					}
					else
					{
					 $valu	=	addslashes(trim($val));
					}
					$formatedValues[] =	"$key = $vala"; $formatedValues1[] = $valu; }
			}
			else
			{
			  $vala	=	"?"; $valu = $val;$formatedValues[]	=	"$key = $vala"; $formatedValues1[] = $valu;
			}
		}
		foreach($conditionValues as $key2 => $val2)
		{
		   $formatedValues1[]	= addslashes(trim($val2));
		}
		
		$sql	=	"UPDATE ".$table." SET ";
		
		$sql	.=	implode(", ",$formatedValues);
		if($strCondition != "") 
		{
			$sql	.=	" WHERE ".$strCondition;
		}
		
		$this->prepareConditionStatement($sql,$formatedValues1);
		return $this->getAffectedRows();
	}
		
	/* prepare statement for delete. parameters passed are : table name, condition for update, array containing values on conditions */
	function prepareDeleteStatement($table,$strCondition='',$conditionValues)
	{
		$sql	=	"DELETE FROM ".$table;
		if($strCondition != "") 
		{
			$sql	.=	" WHERE ".$strCondition;
		}
		$this->prepareConditionStatement($sql,$conditionValues);
		return $this->getAffectedRows();
	}
	// get last inserted ID
	public function lastInsertId()
	{
		return $this->dbh->lastInsertId();
	}
	
	// begin a transaction
	public function beginTransaction()
	{
		return $this->dbh->beginTransaction();
	}
	// end d a transaction
	public function endTransaction()
	{
		return $this->dbh->commit();
	}
	
	// cancel  transaction
	public function cancelTransaction()
	{
		return $this->dbh->rollBack();
	}
	
	//returns number of rows affected on select, update or delete
	public function getAffectedRows()
	{
		
		return $this->stmt->rowCount();
	}
	//close a prepare statement
	public function closeStatement()
	{
		$this->stmt->close();
	}
	// escape quotes
	public function quote($string)
	{
		return $this->dbh->quote($string);
	}
	
	// fetching a single record from the provided table with necessary condition
	public function getTableRecordSingle($tableName,$condition,$conditionValue=array())
	{
		$checkQry	=	"SELECT * FROM $tableName WHERE $condition";
		$result 		= 	$this->resultSetSingle($checkQry,$conditionValue); 
		return $result;
	}
	
	// fetching whole result set from the provided table with necessary condition
	public function getTableRecordDetails($tableName,$condition,$conditionValue)
	{
		$checkQry		=	"SELECT * FROM $tableName WHERE $condition";
		$result	 		= 	$this->resultSetAll($checkQry,$conditionValue); 
		return $result;
	}
	
	// fetching a single record from the provided table with necessary condition and return value of required table field	
	public function getTableRecordField($tableName,$condition,$conditionValue=array(),$returnValue)
	{
		$checkQry	=	"SELECT * FROM $tableName WHERE $condition";
		$result 		= 	$this->resultSetSingle($checkQry,$conditionValue); 
		return trim(stripslashes($result[$returnValue]));
	}
	
	public function getTableRecordCount($tableName,$condition,$conditionValues=array())
	{
		$query		=	"SELECT * FROM $tableName WHERE $condition";
		$this->prepareConditionStatement($query,$conditionValues);
		return $this->getAffectedRows();
	}
	
	public function getRequiredFieldsSingle($tableName,$condition,$conditionValue=array(),$requiredFields)
	{
		$checkQry		=	"SELECT $requiredFields FROM $tableName WHERE $condition";
		$result 		= 	$this->resultSetSingle($checkQry,$conditionValue); 
		return $result;
	}
	
	public function getRequiredFieldsAll($tableName,$fieldName,$condition,$conditionValue=array())
	{
		$checkQry		=	"SELECT $fieldName FROM $tableName WHERE $condition";
		$result	 		= 	$this->resultSetAll($checkQry,$conditionValue); 
		return $result;
	}
	
	public function getTableRecordDetailsWithCondition($tableName,$condition,$conditionValue=array(),$orderCondn='',$limitCondn='')
	{
		$checkQry		=	"SELECT * FROM $tableName WHERE $condition ".$orderCondn.' '.$limitCondn;
		$result	 		= 	$this->resultSetAll($checkQry,$conditionValue); 
		return $result;
	}
	
	public function getTableRecordNoCodition($tableName,$orderCondn='',$limitCondn='')
	{
		$checkQry		=	"SELECT * FROM $tableName WHERE 1 ".$orderCondn.' '.$limitCondn;
		$result	 		= 	$this->resultSetAll($checkQry); 
		return $result;
	}
	
	function checkEntryExist($table,$condition,$conditionValues)
	{
		$flag = false;
		$checkQry	=	"SELECT * FROM $table WHERE $condition";
		$this->prepareConditionStatement($checkQry,$conditionValues);
		$numRows = $this->getAffectedRows();
		if($numRows>0) $flag = true;
		return $flag;
	}
	
	public function getSingleFieldDetails($tableName,$fieldName,$condition,$conditionValue)
	{
		$checkQry		=	"SELECT $fieldName FROM $tableName WHERE $condition";
		$result	 		= 	$this->resultSetAll($checkQry,$conditionValue); 
		return $result;
	}




function backup_tables_each($tables,$compression,$format) 
{
	$this->dbh->setAttribute(PDO::ATTR_ORACLE_NULLS, PDO::NULL_NATURAL );
	//Script Variables
	$BACKUP_PATH = "DB_DUMP";
	$date = date("Y-m-d");
	$olddir =  getcwd();
	chdir($BACKUP_PATH);
	if(!file_exists($date)) {
		echo "<font color=red>Created '".$date."' folder</font><br />";
		$cmd = "mkdir ".$date;
		exec($cmd);
		}
	chdir($date);
	
	//array of all database field types which just take numbers 
	$numtypes = array('tinyint', 'smallint', 'mediumint', 'int', 'bigint', 'float', 'double', 'decimal', 'real');
	
	//get all of the tables
	if(empty($tables)) {
		$pstm1 = $this->dbh->query('SHOW TABLES');
		while($row = $pstm1->fetch(PDO::FETCH_NUM)) {
			$tables[] = $row[0];
			}
		}
	else {
		$tables = is_array($tables) ? $tables : explode(',',$tables);
		}
	
	//cycle through the table(s)
	echo "<font color=blue>Dumping tables to DB_DUMP:</font>";
	echo "<ul>";
	foreach($tables as $table) {
		//create/open files
		if($format == "csv") {
			$filename = $table.".csv";
			$handle = fopen($filename,"w");
			}
		else {
			if($compression) {
				$filename = $table.".sql.gz";
				$zp = gzopen($filename,"wb9");
				}
			else {
				$filename = $table.".sql";
				$handle = fopen($filename,"w");
				}
			}
		echo "<li><small><font color=blue>".$filename."</font>";
		$result = $this->dbh->query("SELECT * FROM $table");
		$num_fields = $result->columnCount();
		$num_rows = $result->rowCount();
		$return = "";
		$return .= 'DROP TABLE IF EXISTS `'.$table.'`;'; 
	
		//table structure
		$pstm2 = $this->dbh->query("SHOW CREATE TABLE $table");
		$row2 = $pstm2->fetch(PDO::FETCH_NUM);
		$ifnotexists = str_replace('CREATE TABLE', 'CREATE TABLE IF NOT EXISTS', $row2[1]);
		$return .= "\n\n".$ifnotexists.";\n\n";
	
		if($format <> "csv") {
			if($compression) gzwrite($zp, $return);
			else fwrite($handle,$return);
			}
		$return = "";
	
		//insert values
		if($num_rows) {
			$return = 'INSERT INTO `'."$table"."` (";
			$pstm3 = $this->dbh->query("SHOW COLUMNS FROM $table");
			$count = 0;
			$type = array();
	
			while($rows = $pstm3->fetch(PDO::FETCH_NUM)) {
				if(stripos($rows[1], '(')) {
					$type[$table][] = stristr($rows[1], '(', true);
					}
				else $type[$table][] = $rows[1];
	
				$return .= "`".$rows[0]."`";
				$count++;
				if($count < ($pstm3->rowCount())) {
					$return .= ", ";
					}
				}
			$return .= ")".' VALUES';
			if($format <> "csv") {
				if($compression) gzwrite($zp, $return);
				else fwrite($handle,$return);
				}
			$return = "";
			}
		$count = 0;
		while($row = $result->fetch(PDO::FETCH_NUM)) {
			if($format <> "csv") $return = "\n\t(";
			for($j=0; $j < $num_fields; $j++) {
				//$row[$j] = preg_replace("\n","\\n",$row[$j]);
				if(isset($row[$j])) {
					if($format == "csv") $return .= '"'.$row[$j].'"';
					else {
						//if number, take away "". else leave as string
						if((in_array($type[$table][$j],$numtypes)) && (!empty($row[$j])))
							$return .= $row[$j];
						else
							$return .= $this->dbh->quote($row[$j]);
						}
					}
				else {
					if($format == "csv") $return .= '';
					else $return .= 'NULL';
					}
				if($j < ($num_fields-1)) $return .= ',';
				}
			$count++;
			if($format == "csv") $return .= "\n";
			else {
				if($count < ($result->rowCount()))
					$return .= "),";
				else $return .= ");";
				}
			if($format == "csv") fwrite($handle,$return);
			else {
				if($compression) gzwrite($zp, $return);
				else fwrite($handle,$return);
				}
			$return = "";
			}
		$return = "\n\n-- ------------------------------------------------ \n\n";
		echo " (".$count." records)";
	
		if($format <> "csv") {
			if($compression) gzwrite($zp, $return);
			else fwrite($handle,$return);
			}
		$return = "";
	
		$error1 = $pstm2->errorInfo();
		$error2 = $pstm3->errorInfo();
		$error3 = $result->errorInfo();
		echo $error1[2];
		echo $error2[2];
		echo $error3[2];
	
		if($format == "csv") fclose($handle);
		else {
			if($compression) gzclose($zp);
			else fclose($handle);
			}
		$filesize = filesize($filename);
		echo " - ".$filesize." bytes</small></li>";
		}
	echo "</ul>";
	chdir($olddir);
	return;
	}
	
	
	
	function backup_tables($tables,$compression,$format) 
{
	$this->dbh->setAttribute(PDO::ATTR_ORACLE_NULLS, PDO::NULL_NATURAL );
	//Script Variables
	$BACKUP_PATH = "DB_DUMP";
	$date = date("Y-m-d");
	$olddir =  getcwd();
	chdir($BACKUP_PATH);
	if(!file_exists($date)) {
		echo "<font color=red>Created '".$date."' folder</font><br />";
		$cmd = "mkdir ".$date;
		exec($cmd);
		}
	chdir($date);
	
	//array of all database field types which just take numbers 
	$numtypes = array('tinyint', 'smallint', 'mediumint', 'int', 'bigint', 'float', 'double', 'decimal', 'real');
	
	//get all of the tables
	if(empty($tables)) {
		$pstm1 = $this->dbh->query('SHOW TABLES');
		while($row = $pstm1->fetch(PDO::FETCH_NUM)) {
			$tables[] = $row[0];
			}
		}
	else {
		$tables = is_array($tables) ? $tables : explode(',',$tables);
		}
	
	//cycle through the table(s)
	echo "<font color=blue>Dumping tables to DB_DUMP:</font>";
	echo "<ul>";
	
	$return = "";
	$filename = $date.".sql";
	$handle = fopen($filename,"w");
			
	foreach($tables as $table) {
		//create/open files

		echo "<li><small><font color=blue>".$table."</font>";
		$result = $this->dbh->query("SELECT * FROM $table");
		$num_fields = $result->columnCount();
		$num_rows = $result->rowCount();
		
		$return .= 'DROP TABLE IF EXISTS `'.$table.'`;'; 
	
		//table structure
		$pstm2 = $this->dbh->query("SHOW CREATE TABLE $table");
		$row2 = $pstm2->fetch(PDO::FETCH_NUM);
		$ifnotexists = str_replace('CREATE TABLE', 'CREATE TABLE IF NOT EXISTS', $row2[1]);
		$return .= "\n\n".$ifnotexists.";\n\n";

	
		//insert values
		if($num_rows) {
			$return .= 'INSERT INTO `'."$table"."` (";
			$pstm3 = $this->dbh->query("SHOW COLUMNS FROM $table");
			$count = 0;
			$type = array();
	
			while($rows = $pstm3->fetch(PDO::FETCH_NUM)) {
				if(stripos($rows[1], '(')) {
					$type[$table][] = stristr($rows[1], '(', true);
					}
				else $type[$table][] = $rows[1];
	
				$return .= "`".$rows[0]."`";
				$count++;
				if($count < ($pstm3->rowCount())) {
					$return .= ", ";
					}
				}
			$return .= ")".' VALUES';
		
			}
		$count = 0;
		while($row = $result->fetch(PDO::FETCH_NUM)) {
			if($format <> "csv") $return .= "\n\t(";
			for($j=0; $j < $num_fields; $j++) {
				//$row[$j] = preg_replace("\n","\\n",$row[$j]);
				if(isset($row[$j])) {
					if($format == "csv") $return .= '"'.$row[$j].'"';
					else {
						//if number, take away "". else leave as string
						if((in_array($type[$table][$j],$numtypes)) && (!empty($row[$j])))
							$return .= $row[$j];
						else
							$return .= $this->dbh->quote($row[$j]);
						}
					}
				else {
					if($format == "csv") $return .= '';
					else $return .= 'NULL';
					}
				if($j < ($num_fields-1)) $return .= ',';
				}
			$count++;
			if($format == "csv") $return .= "\n";
			else {
				if($count < ($result->rowCount()))
					$return .= "),";
				else $return .= ");";
				}
			}
		$return .= "\n\n-- ------------------------------------------------ \n\n";
		echo " (".$count." records)";
	
	
		$error1 = $pstm2->errorInfo();
		$error2 = $pstm3->errorInfo();
		$error3 = $result->errorInfo();
		echo $error1[2];
		echo $error2[2];
		echo $error3[2];
	
		$filesize = filesize($filename);
		echo "</small></li>";
		}
		
		if($compression) gzwrite($zp, $return);
		else fwrite($handle,$return);
	echo "</ul>";
	chdir($olddir);
	return;
	}
}
?>